// -----------------------------------------------------------------------------
//	NetMessenger.h
// -----------------------------------------------------------------------------

#ifndef __NET_MESSENGER__
#define __NET_MESSENGER__

#include <Messenger.h>

class RBMDAddress;

enum netstatus_t {
	NET_OK = 0,					// nothing wrong
	NET_NO_ERROR = 0,			// nothing wrong
	NET_ERROR,					// generic error

	NET_UNKNOWN_HOST,			// the given hostname is unresolvable
	NET_TIMED_OUT,				// the connection attempt timed out
	NET_CANT_CONNECT,			// the connection request was refused
	NET_CANT_REPLY,				// the message doesn't contain a reply route
	NET_DISCONNECTED,			// the connection was broken

	NET_LOCAL_ERROR,			// something is wrong locally
	NET_NO_DAEMON,				// can't find a running messaging daemon

	NET_INTERNAL_ERROR			// you should never see this one
};


class NetMessenger {		
public:
					NetMessenger(	const char *dest_addr,
									const char *mime_sig,
									team_id team = -1,
									status_t *perr = NULL,
									bigtime_t connect_timeout = B_INFINITE_TIMEOUT );
	
					NetMessenger(	int32 dest_hostaddr,
									const char *mime_sig,
									team_id team = -1,
									status_t *perr = NULL,
									bigtime_t connect_timeout = B_INFINITE_TIMEOUT );

					NetMessenger(	const BMessage *message,
									status_t *perr = NULL,
									bigtime_t connect_timeout = B_INFINITE_TIMEOUT );

					NetMessenger(	const NetMessenger &copy );

	virtual			~NetMessenger();

	NetMessenger &	operator=( const NetMessenger &copy );
	bool			operator==( const NetMessenger &comp ) const;

	netstatus_t		Error();

	netstatus_t		Ping();

	status_t		GetRemoteHostname( char *host );

	status_t		SendMessage(	uint32 command,
									const BHandler *reply_to = NULL );

	status_t		SendMessage(	BMessage *message,
									BHandler *reply_to = NULL,
									bigtime_t timeout = B_INFINITE_TIMEOUT);

private:
	BMessenger		_localMsgr;
	RBMDAddress *	_destination;
	netstatus_t		_currstat;

	netstatus_t		Connect( const char *, bigtime_t );
	netstatus_t		Connect( int32, bigtime_t );
	netstatus_t		RouteReply( RBMDAddress &, const BHandler * ) const;
	void			SetDestination( const RBMDAddress & );

};

#endif	// __NET_MESSENGER__