#include "sampleapp.h"
#include "NetMessenger.h"


const char kSignature[] = "application/x-vnd.Avatar.sampleapp";


sampleapp::sampleapp()
	: BApplication( kSignature ), _sendRequest( true ) {}


thread_id
sampleapp::Run()
{
	status_t status;
	
	try {
		if ( _sendRequest )
		{
			//	To experiment with a networked machine, change the address
			//	here.  Make sure the remote machine's message_daemon settings
			//	file accepts messages from this host to sampleapp.
			NetMessenger msgr( "127.0.0.1", kSignature, -1, &status );
			if ( status != B_OK ) throw msgr.Error();
			BMessage msg( kPerformSummation );
			msg.AddInt32( "n", 20 );
			status = msgr.SendMessage( &msg, this );
			if ( status != B_OK ) throw msgr.Error();
		}
	}
	catch ( netstatus_t stat ) {
//		BAlert *alert = new BAlert( "", "Network error", "OK" );
//		alert->Go();
		PostMessage( B_QUIT_REQUESTED );
	}
	BApplication::Run();
}


void
sampleapp::MessageReceived( BMessage *msg )
{
	switch ( msg->what )
	{
		case kPerformSummation:
			DoSummation( msg );
			break;
		
		case kReceiveResults:
			ShowResults( msg );
			_sendRequest = false;
			break;
		
		default:
			BApplication::MessageReceived( msg );
			break;
	}
}


void
sampleapp::DoSummation( BMessage *msg )
{
	int32 n = msg->FindInt32( "n" );
	int32 sum = ( n * (n + 1) ) / 2;
	status_t status;

	try
	{
		NetMessenger replyMsgr( msg, &status );		// NetMessenger reply ctor
		if ( status != B_OK ) throw status;
		
		BMessage reply( kReceiveResults );
		reply.AddInt32( "sum", sum );
		
		status = replyMsgr.SendMessage( &reply );
		if ( status != B_OK ) throw status;
	}
	catch ( status_t status )
	{
		BAlert *alert = new BAlert( "", "Network error", "OK" );
		alert->Go();
		PostMessage( B_QUIT_REQUESTED );
	}
}


void
sampleapp::ShowResults( BMessage *msg )
{
	int32 sum = msg->FindInt32( "sum" );
	char text[255];
	sprintf( text, "The summation is %d.", sum );
	BAlert *alert = new BAlert( "", (const char *)text, "OK" );
	alert->Go();
	PostMessage( B_QUIT_REQUESTED );
}